/* Header file for GRETA decomposition routines
   Author:  D.C. Radford    Aug 2004
 */

#define BASIS_FILE  "basis.dat"         /* file containing basis data */
#define EVENT_FILE  "input_signals.dat" /* CHANGEME file containing observed data */
#define SUMS_FILE   "precalc.dat"       /* file containing precalculated sums */

#define GRID_PTS   226993  /* CHANGEME number of grid points in the basis */
#define GRID_SEGS  37      /* number of signals calculated for each basis point */
#define MEAS_SEGS  37      /* number of signals measured for each event */
#define TIME_STEPS 50      /* number of time steps calculated/measured for each segment */
#define TOT_SEGS   36      /* total number of segments in crystal */
#define MAX_AGS    550     /* CHANGEME max. no. of points in coarse grid for AGS */

#define MAX_SEGS   8              /* max. number of segments to take in events */
#define MAX_PARS   8*MAX_SEGS+1   /* max. number of parameters to fit */
#define SSEG       TOT_SEGS       /* range of seg in basis grid */
#define SRAD       36             /* range of r in basis grid */
#define SPHI       13             /* range of phi in basis grid */
#define SZZZ       20             /* range of z in basis grid */

/* data structure for measured event signals */
typedef struct {
  float total_energy;                    /* total energy in crystal */
  float seg_energy[TOT_SEGS];            /* net energy in each segment */
  float signal[MEAS_SEGS][TIME_STEPS];   /* actual measured signals */
} Event_Signal;

/* data structure for calculated basis signals */
typedef struct {
  char  iseg, ir, ip, iz;             /* integer cylindrical coordinates of grid point */
  float x, y, z;                      /* cartesian coordinates of grid point */
  float signal[GRID_SEGS][50];        /* actual basis signals */
  int   lo_time[GRID_SEGS], hi_time[GRID_SEGS];     /* limits for non-zero signal */
} Basis_Point;

/* data structure for Adaptive Grid Search coarse grid */
typedef struct {
  int    npts;                /* number of AGS points for this segment */
  int    grid_pos[MAX_AGS];   /* pointer to basis-signal-ID for each AGS point */
  double *da;                 /* precalculated sums */
  float  *db;                 /* precalculated sums */
} Adaptive_Grid;

/* data structure for interactions */
typedef struct {
  int    seg;                 /* segment */
  int    pos;                 /* basis signal position, if applicable, else -1 */
  double r, p, z, e;          /* parameters */
  double dr, dp, dz, de;      /* uncertainties */
} Interaction;

Basis_Point   *basis;                 /* basis-signal data */
int           grid_pos_lu[SSEG][SRAD][SPHI][SZZZ];    /* basis-grid position look-up table */
int           maxir[SSEG], maxip[SSEG], maxiz[SSEG];  /* max. values of ir, ip, iz for each segment */
Adaptive_Grid ags1[SSEG];             /* Adaptive Grid Search coarse grid, 1 segment */
Adaptive_Grid ags2[SSEG];             /* Adaptive Grid Search coarse grid, 2 segments */
float         *ags_dc[SSEG][SSEG];    /* more precalculated sums */

int           quiet;                  /* set to 1 to prevent output of diagnostic info */
